function [A, outliers] = tuckey_GME(opticalFlow, max_nb_iter, tolerance_outliers)
%fonction pour calculer la matrice de déplacement globale de la caméra A et
%retourer l'image binaire des outliers (mouvement hors caméra)
%
%Usage:   [A, outliers] = tuckey_GME(opticalFlow)
%Les arguments 'max_nb_iter' et 'tolerance_outliers' sont optionels.

if nargin <2
    max_nb_iter=1;
end

if nargin ==2
    tolerance_outliers=3;
end

eps = 0.01;
size_max = 200;

size_OF = [size(opticalFlow,1), size(opticalFlow,2)];
resize_factor = max(size_OF)/size_max;

OF_resize = imresize(opticalFlow, 1/resize_factor, 'bilinear');

OF_dx = OF_resize(:,:,1);
OF_dy = OF_resize(:,:,2);

D = zeros(length(OF_dx(:)) * 2, 1);
D(1:2:end) = OF_dx(:);
D(2:2:end) = OF_dy(:);

[Xmesh, Ymesh] = meshgrid([1:size(OF_resize,2)], [1:size(OF_resize,1)]);
H_ones = ones(size(OF_resize,2) * size(OF_resize,1), 1);
H = zeros(length(D), 6);
H(1:2:end, 1) = H_ones;
H(1:2:end, 2) = Xmesh(:);
H(1:2:end, 3) = Ymesh(:);
H(2:2:end, 4) = H_ones;
H(2:2:end, 5) = Xmesh(:);
H(2:2:end, 6) = Ymesh(:);

%first iteration
A = inv(H.' * H) * H.' * D;
R = H * A - D;
Rx = reshape( R(1:2:end) , size(OF_resize,1), size(OF_resize,2));
Ry = reshape( R(2:2:end) , size(OF_resize,1), size(OF_resize,2));
R_abs = sqrt(Rx.^2 + Ry.^2);

C_value_prec = 0;
C_value = 1;
nb_iter = 1;
while abs(C_value - C_value_prec) > eps && nb_iter < max_nb_iter
    C_value_prec = C_value;
    
    % C_value
    C_value = mean(R)+5*std(R);
    C = C_value* ones(length(R),1);
    
    % next iterations
    W = (R.^2 - C.^2).^2 .* double(abs(R) < C_value);
    A = inv(H.' * diag(W) * H) * H.' * D;
    R = H * A - D;
    
    % View R
    Rx = reshape( R(1:2:end) , size(OF_resize,1), size(OF_resize,2));
    Ry = reshape( R(2:2:end) , size(OF_resize,1), size(OF_resize,2));
    
    R_abs = sqrt(Rx.^2 + Ry.^2);
    %     figure(1)
    %     imagesc(R_abs )
    
    % View W
    %     Wx = reshape( W(1:2:end) , size(OF_resize,1), size(OF_resize,2));
    %     Wy = reshape( W(2:2:end) , size(OF_resize,1), size(OF_resize,2));
    %
    %     W_abs = sqrt(Wx.^2 + Wy.^2);
    %     figure(2)
    %     imagesc(W_abs)
    %     drawnow
    
    
    if nb_iter == 1
        disp(['Computing the GME']);
    else
        disp(['C_value_prec= ' num2str(C_value_prec) ', C_value= ' num2str(C_value)])
    end
    
    nb_iter = nb_iter + 1;
end

outliers_tmp = R_abs >= (mean(R_abs(:))+ tolerance_outliers *std(R_abs(:)));
outliers = imresize(outliers_tmp, size_OF, 'bilinear');
% figure(3);
% subplot(121)
% imshow(img1)
% subplot(122)
% imagesc(outliers);