#!/usr/bin/env python
"""
Generate saliency with deep learning model for R,G,B,temporal features.
created by Souad CHAABOUNI in 03/08/2015

"""
import argparse
from google.protobuf import text_format
from sklearn import preprocessing
import numpy as np
import cv2
import cv2.cv as cv
CONST_STRIDE = 5
CONST_PATCH = 100
from scipy import *
import sys
import caffe
import Image
import string
import matplotlib
matplotlib.rcParams['backend'] = "Qt4Agg"
import numpy as np
#import lmdb

def parse_args():
    """Parse input arguments
    """

    parser = argparse.ArgumentParser(description='compute saliency map with deep learning model')

    parser.add_argument('MODEL_FILE',
                        help='Input network model: deploy prototxt file',
			default='./models/bvlc_experience7_4k/deploy.prototxt')
    parser.add_argument('PRETRAINED',
                        help='Input pretrained model: trained caffemodel file',
			default='./models/bvlc_experience7_4k/train/train__iter_13000.caffemodel')
    parser.add_argument('VIDEO_FILE',
                        help='spatial video : rgb video file',
			default='/home/souad/WorkSpace/caffe-master17_04/python/src07_hrc01.yuv.avi')
    parser.add_argument('VIDEO_TEMPO_FILE',
                        help='Temporal video : residual motion video file',
			default='/home/souad/WorkSpace/caffe-master17_04/python/src07_hrc01_tempo.yuv.avi')
    parser.add_argument('OUTPUT_FOLDER',
                        help='Adress of folder will contains the saliency map file',
			default='/home/souad/WorkSpace/caffe-master17_04/python/deepSaliency_withRecouvrement_src07')
    args = parser.parse_args()
    return args

def getGaussianKernelWithCenter(length, center, sigma):
    """ Compute the gaussian kernel of saliency
    """
    auxKernel=cv2.getGaussianKernel(length*3, sigma, cv2.CV_32F)
    gaussianKernel=auxKernel[length + ( length/2-center) : 2*length + ( length/2-center), 0:]
    return gaussianKernel;

def main():
    args = parse_args()
    net = caffe.Classifier(args.MODEL_FILE, args.PRETRAINED)
    cap = cv2.VideoCapture(args.VIDEO_FILE)
    cap_tempo = cv2.VideoCapture(args.VIDEO_TEMPO_FILE)
    caffe_root = './'
    while not cap.isOpened():
        cap = cv2.VideoCapture(args.VIDEO_FILE)
        cv2.waitKey(1000)
        print "Wait for the header"

    while not cap_tempo.isOpened():
        cap_tempo = cv2.VideoCapture(args.VIDEO_TEMPO_FILE)
        cv2.waitKey(1000)
        print "Wait for the header"
    sigma = 50
    nbr_frame = cap.get(cv.CV_CAP_PROP_FRAME_COUNT)
    fps = cap.get( cv.CV_CAP_PROP_FPS)
    width_limit = (int)(cap.get( cv.CV_CAP_PROP_FRAME_WIDTH)/CONST_PATCH) * (int)(CONST_PATCH / CONST_STRIDE )
    height_limit = (int)(cap.get( cv.CV_CAP_PROP_FRAME_HEIGHT)/CONST_PATCH) * (int)(CONST_PATCH / CONST_STRIDE )
    print nbr_frame
    print fps 
    print width_limit, height_limit 
    HEIGHT= (int) (cap.get(cv.CV_CAP_PROP_FRAME_HEIGHT))
    WIDTH = (int) (cap.get(cv.CV_CAP_PROP_FRAME_WIDTH))
  
    pos_frame = cap.get(cv2.cv.CV_CAP_PROP_POS_FRAMES)
    print pos_frame
    array_res = zeros((width_limit, height_limit ))
    while True:
        tab_saliency = np.zeros((HEIGHT, WIDTH), dtype = np.uint8)
        flag, frame = cap.read()
        flag_tempo, frame_tempo = cap_tempo.read()
	kernelXY = np.zeros((HEIGHT, WIDTH), dtype = np.uint8)
        if flag:
            # The frame is ready and already captured
            # cv2.imshow('video', frame)
    	    # LOOP for 
            num_j= 0
            while num_j*CONST_STRIDE+CONST_PATCH < HEIGHT :
               num_i= 0
	       while num_i*CONST_STRIDE+CONST_PATCH < WIDTH :
                    region_rgb = frame[num_j*CONST_STRIDE:num_j*CONST_STRIDE+CONST_PATCH, num_i*CONST_STRIDE:num_i*CONST_STRIDE+CONST_PATCH]
		    region_tempo = frame_tempo[num_j*CONST_STRIDE:num_j*CONST_STRIDE+CONST_PATCH, num_i*CONST_STRIDE:num_i*CONST_STRIDE+CONST_PATCH]

		   #transformation image region 
                    transformer = caffe.io.Transformer({'data': net.blobs['data'].data.shape})
                    transformer.set_transpose('data', (2,0,1))
                    transformer.set_mean('data', np.load(caffe_root + 'data/HOLLYWOOD/LMDB_4k/Hollywood_mean4k.npy').mean(1).mean(1)) # mean pixel
                    transformer.set_raw_scale('data', 255)  # the reference model operates on images in [0,255] range instead of [0,1]
                    transformer.set_channel_swap('data', (2,1,0,3))  # the reference model has channels in BGR order instead of RGB
 		    #region classification
                    region_rgb  = region_rgb .astype(np.float32)
                    region_tempo = region_tempo.astype(np.float32)
		    region = np.concatenate((region_rgb, region_tempo), axis=2)
                    region = region.astype(np.float32)
                    net.blobs['data'].data[...] = transformer.preprocess('data', region)
                    out = net.forward()
                    array_res[num_i][num_j] = out['prob'][0][out['prob'][0].argmax(axis=0)] * out['prob'][0].argmax(axis=0)
                    print  "num_i=", num_i, "num_j=", num_j, array_res[num_i][num_j],'out_prob=',  out['prob'][0].argmax(axis=0), 'out_prob_tot=',out['prob'][0], 'out_prob_facteur=',out['prob'][0][out['prob'][0].argmax(axis=0)]
		    if array_res[num_i][num_j] > 0:
			   for prob_i in range(0, int(10 * array_res[num_i][num_j])):
				kernelX = getGaussianKernelWithCenter(int(HEIGHT), num_j*CONST_STRIDE+int(CONST_PATCH/2), sigma)
			        kernelY = getGaussianKernelWithCenter(int(WIDTH), num_i*CONST_STRIDE+int(CONST_PATCH/2), sigma)
				kernelXY = kernelXY + kernelX * kernelY.transpose()
                    num_i = num_i +1;
	       num_j = num_j +1;
 	    min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(kernelXY)
	    if max_val == 0 :
		max_val = 1
	    kernelXY=kernelXY*(1/max_val)* 255

	    tab_saliency = np.uint8(kernelXY)
	    img = Image.fromarray(tab_saliency)
	    name_file = '%s/saliency_%04d.png' %(args.OUTPUT_FOLDER, pos_frame) 
	    img.save(name_file)

	    #transition to next frame 
            pos_frame = cap.get(cv2.cv.CV_CAP_PROP_POS_FRAMES)
            print str(pos_frame)+" frames"
        else:
            # The next frame is not ready, so we try to read it again
            cap.set(cv2.cv.CV_CAP_PROP_POS_FRAMES, pos_frame-1)
            print "frame is not ready"
            # It is better to wait for a while for the next frame to be ready
            cv2.waitKey(1000)

        if cv2.waitKey(10) == 27:
            break
        if cap.get(cv2.cv.CV_CAP_PROP_POS_FRAMES) == cap.get(cv2.cv.CV_CAP_PROP_FRAME_COUNT):
            # If the number of captured frames is equal to the total number of frames,
            # we stop
            break
       #net = caffe_pb2.NetParameter()
       #  text_format.Merge(open(args.input_net_proto_file).read(), net)
       # print('Drawing net to %s' % args.output_image_file)
       # caffe.draw.draw_net_to_file(net, args.output_image_file, args.rankdir)


if __name__ == '__main__':
    main()
